﻿using System;
using System.Security.Permissions;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Drawing;
using System.Text;
using System.Xml;

namespace ZZBase
{
    /// <summary>
    /// ZZScope で使用する内部データツリー(XML ベース)
    /// </summary>
    public class ZZTree : ZZClass
    {
        #region 定数
        /// <summary>
        /// ルートタグ文字列
        /// </summary>
        public const String ROOT_TAG = "ZZROOT";

        /// <summary>
        /// ZZObject 配下のクラスを表すタグの Name 属性文字列
        /// </summary>
        public const String ATTR_NAME = "Name";

        /// <summary>
        /// ZZObject 配下のクラスを表すタグの Guid 属性文字列
        /// </summary>
        public const String ATTR_GUID = "id";

        #endregion

        #region プロパティ

        /// <summary>
        /// メインデータベース（暫定）
        /// </summary>
        public ZZDataBase DataBase { get; set; }

        /// <summary>
        /// ツリーを構成するドキュメント本体
        /// </summary>
        public XmlDocument XmlDoc { get; set; }

        /// <summary>
        /// ルートノードへの参照
        /// </summary>
        public XmlNode XmlRoot { get; set; }

        #endregion

        /// <summary>
        /// データ保存用ツリー
        /// </summary>
        public ZZTree()
        {
            // 空の本体の作成
            XmlDoc = new XmlDocument();

            // 空のデータベース作成（暫定）
            DataBase = new ZZDataBase();

            // XML 宣言部の作成
            XmlDeclaration xmld = XmlDoc.CreateXmlDeclaration("1.0", "UTF-8", null);
            XmlDoc.AppendChild(xmld);

            // ルートノードの作成
            XmlElement xmle = XmlDoc.CreateElement(ROOT_TAG);
            AddAttribute(xmle, this);
            XmlDoc.AppendChild(xmle);
            XmlRoot = XmlDoc.DocumentElement;　// ルートノードへの参照の更新
        }

        /// <summary>
        /// ファイルからのロード
        /// </summary>
        /// <param name="FileName">ロードするファイル名</param>
        public void LoadFromFile(String FileName)
        {
            XmlDoc.Load(FileName) ; // 本体のファイルからのロード
            XmlRoot = XmlDoc.DocumentElement;　// ルートノードへの参照の更新
        }

        /// <summary>
        /// ファイルへの保存
        /// </summary>
        /// <param name="FileName">保存先のファイル名</param>
        public void SaveToFile(String FileName)
        {
            XmlDoc.Save(FileName) ;
        }

        /// <summary>
        /// ノードを名前で検索する
        /// </summary>
        /// <param name="tag">検索対象の Tag 名("*"で全て)</param>
        /// <param name="Name">検索対象の Name 属性</param>
        /// <returns>初めに見つかったノード</returns>
        public XmlNode GetNodeByName(String tag,String Name)
        {
            // 指定 Tag 名のノードリストを得る
            XmlNodeList xmlnl = XmlDoc.GetElementsByTagName(tag);
            if (xmlnl.Count > 0)
            {
                // 同名の中ではじめに見つかったノードを返す
                foreach (XmlNode xmln in xmlnl)
                {
                    if (xmln.Attributes[ATTR_NAME].Value == Name)
                    {
                        return xmln;
                    }
                }
            }
            return null;
        }

        /// <summary>
        /// ツリーに ZZObject を追加する
        /// </summary>
        /// <param name="zzo">追加する ZZObject</param>
        /// <param name="tag">親ノードの Tag 名("*"で全て)</param>
        /// <param name="tag">親ノードの Name 属性</param>
        /// <returns>成功したら true</returns>
        public bool AddZZObject(ZZObject zzo,String tag,String Name)
        {
            XmlNode xmln = GetNodeByName(tag, Name);
            if (xmln != null)
            {   
                // XML ツリーに追加
                return this.AddZZObject(zzo,xmln);
            }
            // その名前を持つノードが見つからなかった
            return false;
        }

        /// <summary>
        /// ツリーの ROOT に ZZObject を追加する
        /// </summary>
        /// <param name="zzo">追加する ZZObject</param>
        /// <returns>成功したら true</returns>
        public bool AddZZObject(ZZObject zzo)
        {
            return AddZZObject(zzo, XmlRoot);
        }

        /// <summary>
        /// ツリーに ZZObject を追加する
        /// </summary>
        /// <param name="zzo">追加する ZZObject</param>
        /// <param name="position">親ノード</param>
        /// <returns>成功したら true</returns>
        public bool AddZZObject(ZZObject zzo, XmlNode position)
        {
            //（暫定）DBに追加
            DataBase.MainDB.Add(zzo.GUID.ToString(), zzo);

            // XML ツリーに追加
            position.AppendChild(MakeTreeElement(zzo));
            return true;
        }
        /// <summary>
        /// 現在の XML ツリーからツリービューを更新する
        /// </summary>
        /// <returns>更新成功したら true</returns>
        public bool UpdateTreeView(TreeView tv)
        {
            TreeNode rtn = new TreeNode(XmlRoot.Name);    //TreeNode クラスのインスタンス生成
            tv.Nodes.Clear();               // ノードをすべて消去
            tv.Nodes.Add(rtn);        // ツリーノードのルートノードの作成
            RecursiveAddTree(XmlRoot, rtn);  // ツリーノード化関数呼び出し
            return true;
        }

        #region プライベートメソッド

        /// <summary>
        /// ZZObject の必須属性をツリー要素に変換する
        /// </summary>
        /// <param name="zzo"></param>
        /// <returns></returns>
        private XmlElement MakeTreeElement(ZZObject zzo)
        {
            // クラス名が要素名になる
            XmlElement xmle = XmlDoc.CreateElement(zzo.GetType().Name);

            /// ZZObject の必須属性をツリー要素に変換する
            return AddAttribute(xmle,zzo);
        }

        /// <summary>
        /// 必須属性を追加する
        /// </summary>
        /// <param name="xmle">追加対象 XmlElement</param>
        /// <param name="zzo">ZZObject</param>
        /// <returns>追加対象 XmlElement</returns>
        private XmlElement AddAttribute(XmlElement xmle,ZZObject zzo)
        {
            // Name 属性の付与
            xmle.SetAttribute(ATTR_NAME, zzo.Name);

            // Guid の付与
            xmle.SetAttribute(ATTR_GUID, zzo.GUID.ToString());

            return xmle;
        }

        /// <summary>
        /// ツリーにツリーノードを追加する
        /// ＜参照＞
        /// MSDN：Visual C# による XML チュートリアル
        /// 第 3 回：Visual C# による簡易 XML エディタ
        /// http://www.microsoft.com/japan/msdn/academic/Articles/xml/03/xml3.aspx
        /// </summary>
        /// <param name="Parentnode">ツリー化を開始するXMLノード</param>
        /// <param name="ParenttreeNode">ツリー化を開始するTreeViewNode</param>
        private void RecursiveAddTree(XmlNode Parentnode, TreeNode ParenttreeNode)
        {
            // 参照を付加
            ParenttreeNode.Tag = Parentnode;
            foreach (XmlNode childXmlNode in Parentnode.ChildNodes)
            {
                // ツリーノードの追加
                TreeNode childTreeNode = ParenttreeNode.Nodes.Add(childXmlNode.Attributes[ATTR_NAME].Value);
                RecursiveAddTree(childXmlNode, childTreeNode); // 再帰呼び出し
            }
        }

        #endregion
    }
}
