﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
//using System.Linq;
using System.Text;

using ZZBase; // ZZ関連の基本クラスライブラリ

namespace ZZComm
{
    /// <summary>
    /// ZZ 関連通信用クラス
    /// 
    /// 送受信などの非同期イベントは ZZObject の 
    /// public EventHandler OnEvent() ;
    /// を使用。個々のイベント通知については派生クラスに依存。
    /// 
    /// なお、OnEvent() は ZZComm の派生クラスの実装によっては、
    /// UI スレッドとは別のスレッドから呼ばれる可能性があるので、
    /// UI(Form コンポーネント) をイベントハンドラの中で取り扱う場合は
    /// 必ず UI スレッドの Invoke() を使用しなければいけない。
    /// </summary>
    public abstract class ZZComm : ZZClass
    {
        /// <summary>
        /// パラメータ記憶用ハッシュ
        /// ＜パラメータ名 -（実体,Set時Hook,Get時Hook）＞の空の辞書
        /// 書き込み時、読み出し時のフックルーチンサポートつき。
        /// フックルーチンは ZZCommParam に対して delegate 名称の形で与える。
        /// </summary>
        [CategoryAttribute("通信ポート")]
        [Description("ポートに付随するパラメータです。")]
        public Dictionary<String,ZZCommParam> Param { set; get ; }

        /// <summary>
        /// パラメータにアクセスするためのインデクサ
        /// </summary>
        /// <param name="s">パラメータ名</param>
        /// <returns>値</returns>
        public String this[String s]
        {
            get
            {
                try
                {
                    return Param[s].Val;
                }
                catch
                {
                    // デフォルトパラメータ辞書に無い場合 Exception が起こる
                    return null;
                }
            }
            set
            {
                // デフォルトパラメータ辞書に無い場合 Exception が起こる
                ZZCommParam zcp = Param[s];
                zcp.Val = value;
            }
        }

        /// <summary>
        /// 名前つきコンストラクタ
        /// </summary>
        /// <param name="s">ノード名称</param>
        public ZZComm(String s)
            : base(s)
        { }

        /// <summary>
        /// 通信ポートのオープン
        /// 失敗の場合は ZZCommException が発生する
        /// </summary>
        public abstract void Open();

        /// <summary>
        /// 通信ポートのクローズ
        /// </summary>
	    public abstract void Close();			

        /// <summary>
        /// データの送信
        /// エラーが発生した場合は ZZCommException が発生する
        /// </summary>
        /// <param name="DataBlock">送信したいデータ</param>
        /// <returns>送信できたバイト数</returns>
        public abstract int Send(byte[] DataBlock);

        /// <summary>
        /// 文字列データの送信
        /// </summary>
        /// <param name="s">送信したい文字列データ</param>
        /// <returns>送信できたバイト数</returns>
        public virtual int SendStr(String s)
        {
            return Send(ZZUtil.StrToByte(s));
        }
				
        /// <summary>
        /// データの受信
        /// エラーが発生した場合は ZZCommException が発生する
        /// </summary>
        /// <param name="DataBlock">受信したデータ</param>
        /// <returns>受信できたバイト数</returns>
        public abstract int Recv(out byte[] DataBlock) ;

        /// <summary>
        /// 文字列として受信
        /// </summary>
        /// <returns>受信文字列そのもの</returns>
        public virtual string RecvStr()
        {
            byte[] ba;
            Recv(out ba);
            return ZZUtil.ByteToStr(ba);
        }

        /// <summary>
        /// 通信オブジェクトでのコマンド実行
        /// エラーが発生した場合は ZZCommException が発生する
        /// </summary>
        /// <param name="CommName">コマンド名</param>
        /// <param name="data">引数列</param>
        /// <returns>失敗した場合 false</returns>
        public virtual bool ExecCommand(String CommName, Object[] data)
        {
            // まだなにもサポートしていない
            return false ;
        }
    }
}
