﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Text;

namespace ZZComm
{
    /// <summary>
    /// パラメータとセット、ゲット時フックを保持するコンテナ
    /// </summary>
    [TypeConverter(typeof(ExpandableObjectConverter))]
    public class ZZCommParam
    {
        /// <summary>
        /// デフォルトコンストラクタ
        /// 初期値のない実体をつくらせない
        /// </summary>
        private ZZCommParam() 
        {
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="v">パラメータ初期値</param>
        /// <param name="g">パラメータ取得時Hook</param>
        /// <param name="s">パラメータセット時Hook</param>
        public ZZCommParam(String v, ParamHook g, ParamHook s) 
            : this()
        {
            // 初期値のセット
            _val = v;

            // 読み出し時フックのセット（null のときは初期値）
            if (g != null)
            {
                getHook = g;
            }

            // 書き込み時フックのセット（null のときは初期値）
            if (s != null)
            {
                setHook = s;
            }

            // 初期値セット後のフック(setHook)呼び出しは
            // すべての初期値が整うまで（ここでは）行わない。
        }

        /// <summary>
        /// パラメータ Set/Get 時に行うフックメソッドの定義
        /// </summary>
        /// <param name="s">パラメータ</param>
        /// <returns>加工後パラメータ</returns>
        public delegate String ParamHook(String s);

        /// <summary>
        /// パラメータ本体
        /// </summary>
        private String _val;

        /// <summary>
        /// パラメータ本体のプロパティ
        /// </summary>
        [Description("パラメータの値です。")]
        public String Val {
            set
            {
                if (setHook != null)
                {
                    _val = setHook(value);
                }
                else
                {
                    throw new ZZCommException("パラメータ設定時のフックが登録されていません");
                }
            }
            get
            {
                if (getHook != null)
                {
                    return getHook(_val);
                }
                else
                {
                    throw new ZZCommException("パラメータ取得時のフックが登録されていません");
                }
            }
        }

        /// <summary>
        /// パラメータ取得時フック
        /// </summary>
        [ReadOnly(true)]
        [Browsable(false)]
        [Description("パラメータ取得時フックです。")]
        public ParamHook getHook { set; get; }

        /// <summary>
        /// パラメータセット時フック
        /// </summary>
        [ReadOnly(true)]
        [Browsable(false)]
        [Description("パラメータ設定時フックです。")]
        public ParamHook setHook { set; get; }
    }
}
