﻿using System;
using System.IO;
using System.ComponentModel;
using System.Collections.Generic;
using System.IO.Ports;

namespace ZZComm
{
    /// <summary>
    /// .NET で提供されているシリアルポート(System.IO.Ports.SerialPort)を,
    /// ZZComm インタフェースでくるんだクラス
    /// </summary>
    public class ZZCommSerial : ZZComm
    {
        /// <summary>
        /// PORT プロパティをあらわす文字列
        /// </summary>
        public const string PORT_STR = "PORT";

        /// <summary>
        /// BITRATE プロパティをあらわす文字列
        /// </summary>
        public const string BITRATE_STR = "BITRATE";

        /// <summary>
        /// SENDDELIM プロパティをあらわす文字列
        /// </summary>
        public const string SENDDELIM_STR = "SENDDELIM";

        /// <summary>
        /// MAXRXSIZE プロパティをあらわす文字列
        /// </summary>
        public const string MAXRXSIZE_STR = "MAXRXSIZE";

        /// <summary>
        /// LOOPBACK プロパティをあらわす文字列
        /// </summary>
        public const string LOOPBACK_STR = "LOOPBACK";

        /// <summary>
        /// COM ポート文字列(e.g. COM1)
        /// </summary>
        [CategoryAttribute("通信ポート")]
        [Description("COM ポート文字列(e.g. COM1)")]
        public String Port
        {
            get { return this[PORT_STR]; }
            set { this[PORT_STR] = value; }
        }

        /// <summary>
        /// ビットレート(e.g. 9600,19200..)
        /// </summary>
        [CategoryAttribute("通信ポート")]
        [Description("ビットレート(e.g. 9600,19200..)")]
        public String Bitrate
        {
            get { return this[BITRATE_STR]; }
            set { this[BITRATE_STR] = value; }
        }

        /// <summary>
        /// 送信時デリミタ(e.g. "CRLF","CR","LF","NONE")
        /// </summary>
        [CategoryAttribute("通信ポート")]
        [Description("送信時デリミタ(e.g. CRLF, CR ,LF ,NONE)")]        
        public String Senddelim
        {
            get { return this[SENDDELIM_STR] ; }
            set { this[SENDDELIM_STR] = value; }
        }

        /// <summary>
        /// .NET で提供されているシリアルポート
        /// </summary>
        private SerialPort port = new SerialPort() ;

        /// <summary>
        /// ループバック用バッファ
        /// </summary>
        private MemoryStream ms = new MemoryStream();

        /// <summary>
        /// デフォルトコンストラクタ
        /// </summary>
        public ZZCommSerial(String Name)
            : base(Name)
        {
            // デフォルトパラメータの設定
            Param = new Dictionary<string,ZZCommParam>()
            {
                // (例)
                // パラメータの説明
                // {"パラメータ名",new ZZCommParam("初期値",読み出し時フック,書き込み時フック)},

                // ポート番号
                {PORT_STR     ,new ZZCommParam("COM1",(m => m),SetPortName)},
                // ビットレート
                {BITRATE_STR  ,new ZZCommParam("9600",(m => m),SetBitRate)},
                // 送信時のデリミタ
                {SENDDELIM_STR ,new ZZCommParam("NONE",(m => m),SetSendDelim)},
                // 一度に受信する最大バイト数
                {MAXRXSIZE_STR ,new ZZCommParam("32768",(m => m),SetReadBufferSize)},
                // ループバックテスト
                {LOOPBACK_STR ,new ZZCommParam("false",(m => m),(m => m))},
            };

            // 初期値を一旦セット(フックルーチンをコール)する
            foreach (String s in Param.Keys)
            {
                Param[s].setHook(this[s]);
            }

            // 受信イベントハンドラの登録
            port.DataReceived += SerialPort_DataReceived;
        }

        /// <summary>
        /// ポート名とビットレートだけのコンストラクタ
        /// </summary>
        /// <param name="Name">表示名</param>
        /// <param name="port">ポート文字列</param>
        /// <param name="bitrate">ビットレート</param>
        public ZZCommSerial(String Name,String port, int bitrate) 
            : this(Name)
        {
            Port = port;
            Bitrate = bitrate.ToString();
        }

        /// <summary>
        /// ポート番号とビットレートと送信デリミタ指定ありのコンストラクタ
        /// </summary>
        /// <param name="Name">表示名</param>
        /// <param name="port">ポート文字列</param>
        /// <param name="bitrate">ビットレート</param>
        /// <param name="senddelim">送信デリミタ</param>
        public ZZCommSerial(String Name,String port, int bitrate,String senddelim)
            : this(Name,port,bitrate)
        {
            Senddelim = senddelim;
        }

        /// <summary>
        /// ループバックモードかどうか
        /// </summary>
        public bool IsLoopBackMode()
        {
            return (this[LOOPBACK_STR].ToLower() == "true");
        }

        /// <summary>
        /// ポートを開ける
        /// </summary>
        public override void Open()
        {
            // シリアルポートのオープン
            if (!IsLoopBackMode())
            {
                port.Open();
            }
        }

        /// <summary>
        /// ポートを閉じる
        /// </summary>
        public override void Close()
        {
            // シリアルポートのクローズ
            if (!IsLoopBackMode())
            {
                port.Close();
            }
        }

        /// <summary>
        /// シリアルポートからの送信
        /// </summary>
        /// <param name="DataBlock">送信データ</param>
        /// <returns>送信できたバイト数（現状では引数から生成）</returns>
        public override int Send(byte[] DataBlock)
        {
            // ループバックテストでない場合
            byte[] Delim = ZZCommDelim.Kind[this[SENDDELIM_STR]];
            if (!IsLoopBackMode())
            {
                // データ本体の送信
                port.Write(DataBlock, 0, DataBlock.Length);

                // デリミタの送信
                port.Write(Delim, 0, Delim.Length);
            }
            else
            {
                // データ本体の送信
                ms.Write(DataBlock, 0, DataBlock.Length);

                // デリミタの送信
                ms.Write(Delim, 0, Delim.Length);

                // 受信イベントの発生
                SerialPort_DataReceived(this, null);
            }
            return DataBlock.Length + Delim.Length;
        }

        /// <summary>
        /// シリアルポートからの受信（同期だがブロックしない）
        /// 最大受信バッファサイズ以上の受信が見込まれる場合は、
        /// 繰り返し呼び出すこと。
        /// </summary>
        /// <param name="DataBlock">受信データ(最大バッファサイズまでで打ち切り)</param>
        /// <returns>受信できたバイト数(データがない時は 0)</returns>
        public override int Recv(out byte[] DataBlock)
        {
            int result;

            // 受信データがあるか？
            if (!IsLoopBackMode())
            {
                if (port.BytesToRead > 0)
                {
                    // ポートからの受信
                    DataBlock = new byte[port.ReadBufferSize];
                    result = port.Read(DataBlock, 0, port.ReadBufferSize);

                    // 受信バイト数分切り詰めて返す
                    Array.Resize<byte>(ref DataBlock, result);
                }
                else
                {
                    // 受信データがない
                    result = 0;
                    DataBlock = null;
                }
            }
            else
            {
                if (ms.Length > 0)
                {
                    // メモリストリームからの受信
                    lock(ms)
                    {
                        DataBlock = new byte[ms.Length];
                        ms.Seek(0, SeekOrigin.Begin);
                        result = ms.Read(DataBlock, 0, (int)ms.Length);
                        ms.Flush();
                        ms.Dispose();
                        ms = new MemoryStream();
                    }
                }
                else
                {
                    // 受信データがない
                    result = 0;
                    DataBlock = null;
                }
            }
            return result;
        }

        /// <summary>
        /// 読み込みバッファサイズの設定(SerialPort オブジェクトに対して)
        /// </summary>
        /// <param name="s">バッファサイズ文字列</param>
        private String SetReadBufferSize(String s)
        {
            int result ;
            if (int.TryParse(s, out result))
            {
                port.ReadBufferSize = result;
            }
            return port.ReadBufferSize.ToString();
        }

        /// <summary>
        /// 送信デリミタの設定(SerialPort オブジェクトに対して)
        /// </summary>
        /// <param name="s">送信デリミタ文字列</param>
        private String SetSendDelim(String s)
        {
            byte[] Delim ;
            try
            {
                // 定義にあるか、検索してみる
                Delim = ZZCommDelim.Kind[s];
                return s;
            }
            catch
            {
                // 定義にない場合は変更しない
                return this[SENDDELIM_STR] ;
            }
        }

        /// <summary>
        /// ビットレートの設定(SerialPort オブジェクトに対して)
        /// </summary>
        /// <param name="s">ビットレート文字列</param>
        private String SetBitRate(String s)
        {
            bool portisopend = port.IsOpen ;

            // いったんクローズする。
            if (portisopend)
            {
                port.Close();
            }

            // ビットレートをセット
            int result;
            if (int.TryParse(s,out result))
            {
                port.BaudRate = result;
            }

            // 再オープン
            if (portisopend)
            {
                port.Open();
            }

            return port.BaudRate.ToString();
        }

        /// <summary>
        /// ポートの設定(SerialPort オブジェクトに対して)
        /// </summary>
        /// <param name="s">ポート文字列</param>
        private String SetPortName(String s)
        {
            bool portisopend = port.IsOpen;

            // いったんクローズする。
            if (portisopend)
            {
                port.Close();
            }

            // ポートをセット
            port.PortName = s;

            // 再オープン
            if (portisopend)
            {
                port.Open();
            }

            return port.PortName.ToString();
        }

        /// <summary>
		/// シリアルポート受信イベントです。
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
        private void SerialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            // ZZObject 自身のイベントハンドラに移譲
            OnEvent(this, e);
        }
    }
}
